@@ -1,7 +1,7 @@
 # cython.* namespace for pure mode.
 from __future__ import absolute_import
 
-__version__ = "0.28.5"
+__version__ = "0.24.1"
 
 try:
     from __builtin__ import basestring
@@ -109,27 +109,22 @@ cclass = ccall = cfunc = _EmptyDecoratorAndManager()
 
 returns = wraparound = boundscheck = initializedcheck = nonecheck = \
     overflowcheck = embedsignature = cdivision = cdivision_warnings = \
-    always_allows_keywords = profile = linetrace = infer_types = \
+    always_allows_keywords = profile = linetrace = infer_type = \
     unraisable_tracebacks = freelist = \
-        lambda _: _EmptyDecoratorAndManager()
-
-exceptval = lambda _=None, check=True: _EmptyDecoratorAndManager()
+        lambda arg: _EmptyDecoratorAndManager()
 
 optimization = _Optimization()
 
 overflowcheck.fold = optimization.use_switch = \
     optimization.unpack_method_calls = lambda arg: _EmptyDecoratorAndManager()
 
-final = internal = type_version_tag = no_gc_clear = no_gc = _empty_decorator
+final = internal = type_version_tag = no_gc_clear = _empty_decorator
 
 
-_cython_inline = None
 def inline(f, *args, **kwds):
     if isinstance(f, basestring):
-        global _cython_inline
-        if _cython_inline is None:
-            from Cython.Build.Inline import cython_inline as _cython_inline
-        return _cython_inline(f, *args, **kwds)
+        from Cython.Build.Inline import cython_inline
+        return cython_inline(f, *args, **kwds)
     else:
         assert len(args) == len(kwds) == 0
         return f
@@ -146,7 +141,6 @@ def cdiv(a, b):
     q = a / b
     if q < 0:
         q += 1
-    return q
 
 def cmod(a, b):
     r = a % b
@@ -385,7 +379,7 @@ py_complex = typedef(complex, "double complex")
 int_types = ['char', 'short', 'Py_UNICODE', 'int', 'Py_UCS4', 'long', 'longlong', 'Py_ssize_t', 'size_t']
 float_types = ['longdouble', 'double', 'float']
 complex_types = ['longdoublecomplex', 'doublecomplex', 'floatcomplex', 'complex']
-other_types = ['bint', 'void', 'Py_tss_t']
+other_types = ['bint', 'void']
 
 to_repr = {
     'longlong': 'long long',
@@ -420,17 +414,14 @@ for name in complex_types:
     gs[name] = typedef(py_complex, to_repr(name, name))
 
 bint = typedef(bool, "bint")
-void = typedef(None, "void")
-Py_tss_t = typedef(None, "Py_tss_t")
+void = typedef(int, "void")
 
 for t in int_types + float_types + complex_types + other_types:
     for i in range(1, 4):
-        gs["%s_%s" % ('p'*i, t)] = gs[t]._pointer(i)
+        gs["%s_%s" % ('p'*i, t)] = globals()[t]._pointer(i)
 
-NULL = gs['p_void'](0)
-
-# looks like 'gs' has some users out there by now...
-#del gs
+void = typedef(None, "void")
+NULL = p_void(0)
 
 integral = floating = numeric = _FusedType()
 
